;*****************************************************************************
;   SCAN.ASM -- ROUTINE TO SET D/A VOLTAGES AND EXERCISE CCD DRIVER OP-AMPS
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4

;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000100                 ; program start location
DZ          EQU     $001000                 ; DAC zero volt offset

IPRC        EQU     $FFFFFF                 ; Interrupt priority register (core)
IPRP        EQU     $FFFFFE                 ; Interrupt priority register (periph)
PCTL        EQU     $FFFFFD                 ; PLL control register
BCR         EQU     $FFFFFB                 ; Bus control register (wait states)
AAR0        EQU     $FFFFF9                 ; Address attribute register 0
AAR1        EQU     $FFFFF8                 ; Address attribute register 1
AAR2        EQU     $FFFFF7                 ; Address attribute register 2
AAR3        EQU     $FFFFF6                 ; Address attribute register 3
TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register 0
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A
PCRC        EQU     $FFFFBF                 ; Port C (ESSI_0) control register
PCRD        EQU     $FFFFAF                 ; Port D (ESSI_1) control register
PRRD        EQU     $FFFFAE                 ; Port D (ESSI_1) direction
PDRD        EQU     $FFFFAD                 ; Port D (ESSI_1) data

SEQREG      EQU     $FFFF80                 ; external CCD clock register
SIG_AB      EQU     $FFFF88                 ; bias voltages A+B
CLK_AB      EQU     $FFFF90                 ; clock voltages A+B

OD_L        EQU     $FFFF8F                 ; OD_L
OD_R        EQU     $FFFF8E                 ; OD_R
RD          EQU     $FFFF8D                 ; RD
SPARE       EQU     $FFFF8C                 ; SPARE
OG          EQU     $FFFF8B                 ; OG
TEC         EQU     $FFFF8A                 ; TEC
OFF_L       EQU     $FFFF89                 ; OFFSET_L
OFF_R       EQU     $FFFF88                 ; OFFSET_R

WAIT        EQU     $073FE1                 ; 1 PERIPH 1 SRAM 31 EPROM
WAIT5       EQU     $07BFE1                 ; 5 PERIPH 1 SRAM 31 EPROM

;*****************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*****************************************************************************

            ORG     P:$0
            JMP     START

;*****************************************************************************
;   MAIN PROGRAM
;*****************************************************************************
            ORG     P:START
SET_MODE    MOVEP   #$FFFC21,X:AAR3         ; PERIPH $FFF000--$FFFFFF
            MOVEP   #$D08909,X:AAR1         ; EEPROM $D00000--$D07FFF 32K
            MOVEP   #$000811,X:AAR0         ; SRAM   $000000--$00FFFF 64K

PORTD_SETUP MOVEP   #>$0,X:PCRD             ; define PD0=TM PD1=GAIN
            MOVEP   #>$3,X:PRRD             ; as GPIO outputs
            MOVEP   #>$0,X:PDRD             ; clear port D

DAC_SETUP   MOVEP   #WAIT5,X:BCR            ;[ ] add wait states
            NOP

            MOVEP   #(DZ+0500),Y:<<OD_L     ; set bias voltages
            MOVEP   #(DZ+0500),Y:<<OD_R     ; (10 mV/DN)
            MOVEP   #(DZ+0500),Y:<<RD       ;
            MOVEP   #(DZ+0500),Y:<<SPARE    ;

            MOVEP   #(DZ-1000),Y:<<OG       ; set bias voltages
            MOVEP   #(DZ+0020),Y:<<TEC      ; (5 mV/DN)
            MOVEP   #(DZ+1000),Y:<<OFF_L    ;
            MOVEP   #(DZ+1000),Y:<<OFF_R    ;


            MOVE    #CLK_AB,R0              ; set clock voltages
            MOVE    #>(DZ+500),X0           ;
            MOVE    #>(DZ-500),Y0

            DO      #$10,END_SET            ;
            MOVE    Y0,Y:(R0)+              ;
            MOVE    X0,Y:(R0)+              ;
END_SET     MOVEP   #WAIT,X:BCR             ;[ ] re-set wait states


MAIN        MOVEP   #>$0AAAAA,Y:<<SEQREG    ; exercise the clocks
            REP     #$FF                    ; small delay
            NOP
            MOVEP   #>$055555,Y:<<SEQREG
            REP     #$FF                    ; small delay
            NOP
            JMP     MAIN

            END

