;*****************************************************************************
;   SLINK.ASM -- SERIAL LINK TEST PROGRAM
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000100                 ; program start location
TXREG       EQU     $FFFF85                 ; Transmit Data Register
RXREG       EQU     $FFFF86                 ; Receive Data register
PDRB        EQU     $FFFFC9                 ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8                 ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4                 ; Port B (HOST) control register
PCRC        EQU     $FFFFBF                 ; Port C (ESSI0) control register
PRRC        EQU     $FFFFBE                 ; Port C (ESSI0) direction
PDRC        EQU     $FFFFBD                 ; Port C (ESSI0) data

TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register 0
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A

PCRD        EQU     $FFFFAF                 ; Port D (ESSI1) control register
PRRD        EQU     $FFFFAE                 ; Port D (ESSI1) direction
PDRD        EQU     $FFFFAD                 ; Port D (ESSI1) data
PCTL        EQU     $FFFFFD                 ; PLL control register

LINK        EQU     $0                      ; link parameter

;*****************************************************************************
;   COMPILE-TIME OPTIONS
;*****************************************************************************
    IF (SYS==0)                             ; 
INIT_LINK       EQU     $0                  ; 0=wire 1=single_fiber
    ENDIF

    IF (SYS==1)                             ; 
INIT_LINK       EQU     $1                  ; 0=wire 1=single_fiber
    ENDIF

;*****************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*****************************************************************************
            ORG     P:$0
            JMP     START

;*****************************************************************************
;   MAIN PROGRAM
;*****************************************************************************
            ORG     P:START
PLL_SETUP   MOVEP   #$078007,X:PCTL         ; set PLL parameters

PORTB_SETUP MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO

SSI_SETUP   MOVEP   #>$032070,X:CRB         ; async, LSB, enable TE RE
            MOVEP   #>$140803,X:CRA         ; 10 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI

PORTD_SETUP MOVEP   #>$0,X:PCRD             ; set PD[5..0] GPIO
            MOVEP   #>$F,X:PRRD             ; TM GAIN /ENRX /ENTX RXRDY
            MOVEP   #>$0,X:PDRD             ; 

            MOVE    #>INIT_LINK,X0          ;
            MOVE    X0,X:LINK               ;

MAIN        JSR     READ16                  ;
            REP     #$28                    ;
            NOP                             ;

            JSR     WRITE16                 ;
            REP     #$28                    ;
            NOP                             ;
            JMP     MAIN                    ;

;*******************************************************************************
;   READ AND WRITE 16-BIT DATA
;*******************************************************************************
READ16      JCLR    #$0,X:LINK,RD16B        ; check RS485 or fiber
            JCLR    #$4,X:PDRD,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A               ; bits 15..0
            AND     #>$FFFF,A               ;
RD16B       JSET    #$0,X:LINK,ENDRD16      ; check RS485 or fiber
            JCLR    #7,X:SSISR,*            ; wait for RDRF to go high
            MOVE    X:RXD,A1                ; read from ESSI
            NOP                             ;
ENDRD16     RTS                             ; 16-bit word in A1

WRITE16     JCLR    #$0,X:LINK,WR16B        ; check RS485 or fiber
            MOVE    A1,Y:TXREG              ; write bits 15..0
WR16B       JSET    #$0,X:LINK,ENDWR16      ;
            JCLR    #6,X:SSISR,*            ; wait for TDE
            MOVE    A1,X:TXD                ;
ENDWR16     RTS                             ;

